@@ -1,9 +1,5 @@
 package aQute.bnd.component;
 
-import static aQute.bnd.component.DSAnnotationReader.V1_0;
-import static aQute.bnd.component.DSAnnotationReader.V1_1;
-import static aQute.bnd.component.DSAnnotationReader.V1_2;
-
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -16,117 +12,110 @@ import java.util.StringTokenizer;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import aQute.bnd.component.annotations.ConfigurationPolicy;
-import aQute.bnd.component.annotations.ReferenceCardinality;
-import aQute.bnd.component.annotations.ReferencePolicy;
-import aQute.bnd.component.annotations.ReferencePolicyOption;
-import aQute.bnd.component.annotations.ServiceScope;
-import aQute.bnd.component.error.DeclarativeServicesAnnotationError;
-import aQute.bnd.component.error.DeclarativeServicesAnnotationError.ErrorType;
-import aQute.bnd.osgi.Analyzer;
-import aQute.bnd.osgi.ClassDataCollector;
-import aQute.bnd.osgi.Clazz;
+import org.osgi.service.component.annotations.ConfigurationPolicy;
+import org.osgi.service.component.annotations.ReferenceCardinality;
+import org.osgi.service.component.annotations.ReferencePolicy;
+import org.osgi.service.component.annotations.ReferencePolicyOption;
+
+import aQute.bnd.osgi.*;
 import aQute.bnd.osgi.Clazz.MethodDef;
-import aQute.bnd.osgi.Constants;
 import aQute.bnd.osgi.Descriptors.TypeRef;
-import aQute.bnd.osgi.Processor;
-import aQute.bnd.osgi.Verifier;
-import aQute.bnd.version.Version;
 import aQute.lib.tag.Tag;
+import aQute.bnd.version.Version;
 
 public class HeaderReader extends Processor {
-	private final static Pattern		PROPERTY_PATTERN	= Pattern
-		.compile("(([^=:@]+)([:@](Boolean|Byte|Char|Short|Integer|Long|Float|Double|String))?)\\s*=(.*)");
-	private final static Set<String>	LIFECYCLE_METHODS	= new HashSet<>(
-		Arrays.asList("activate", "deactivate", "modified"));
+	final static Pattern		PROPERTY_PATTERN		= Pattern
+	.compile("([^=]+([:@](Boolean|Byte|Char|Short|Integer|Long|Float|Double|String))?)\\s*=(.*)");
+	private final static Set<String> LIFECYCLE_METHODS = new HashSet<String>(Arrays.asList("activate", "deactivate", "modified"));
+	
+    private final Analyzer analyzer;
 
-	private final Analyzer				analyzer;
+	private final static String ComponentContextTR = "org.osgi.service.component.ComponentContext";
+	private final static String BundleContextTR = "org.osgi.framework.BundleContext";
+	private final static String MapTR = Map.class.getName();
+	private final static String IntTR = int.class.getName();
+	final static Set<String> allowed = new HashSet<String>(Arrays.asList(ComponentContextTR, BundleContextTR, MapTR));
+	final static Set<String> allowedDeactivate = new HashSet<String>(Arrays.asList(ComponentContextTR, BundleContextTR, MapTR, IntTR));
+	
+	private final static String ServiceReferenceTR = "org.osgi.framework.ServiceReference";
 
-	private final static String			ComponentContextTR	= "org.osgi.service.component.ComponentContext";
-	private final static String			BundleContextTR		= "org.osgi.framework.BundleContext";
-	private final static String			MapTR				= Map.class.getName();
-	private final static String			IntTR				= int.class.getName();
-	final static Set<String>			allowed				= new HashSet<>(
-		Arrays.asList(ComponentContextTR, BundleContextTR, MapTR));
-	final static Set<String>			allowedDeactivate	= new HashSet<>(
-		Arrays.asList(ComponentContextTR, BundleContextTR, MapTR, IntTR));
-
-	private final static String			ServiceReferenceTR	= "org.osgi.framework.ServiceReference";
-
-	public HeaderReader(Analyzer analyzer) {
-		this.analyzer = analyzer;
-	}
-
-	public Tag createComponentTag(String name, String impl, Map<String, String> info) throws Exception {
-		final ComponentDef cd = new ComponentDef(analyzer, null, V1_0);
+    public HeaderReader(Analyzer analyzer) {
+    	this.analyzer = analyzer;
+    }
+    
+	public Tag createComponentTag(String name, String impl, Map<String, String> info)
+	throws Exception {
+		final ComponentDef cd = new ComponentDef();
 		cd.name = name;
 		if (info.get(COMPONENT_ENABLED) != null)
 			cd.enabled = Boolean.valueOf(info.get(COMPONENT_ENABLED));
 		cd.factory = info.get(COMPONENT_FACTORY);
-		if (info.get(COMPONENT_IMMEDIATE) != null)
-			cd.immediate = Boolean.valueOf(info.get(COMPONENT_IMMEDIATE));
+		if (info.get(COMPONENT_IMMEDIATE) != null) 
+		    cd.immediate = Boolean.valueOf(info.get(COMPONENT_IMMEDIATE));
 		if (info.get(COMPONENT_CONFIGURATION_POLICY) != null)
-			cd.configurationPolicy = ConfigurationPolicy.valueOf(info.get(COMPONENT_CONFIGURATION_POLICY)
-				.toUpperCase());
+		    cd.configurationPolicy = ConfigurationPolicy.valueOf(info.get(COMPONENT_CONFIGURATION_POLICY).toUpperCase());
 		cd.activate = checkIdentifier(COMPONENT_ACTIVATE, info.get(COMPONENT_ACTIVATE));
 		cd.deactivate = checkIdentifier(COMPONENT_DEACTIVATE, info.get(COMPONENT_DEACTIVATE));
 		cd.modified = checkIdentifier(COMPONENT_MODIFIED, info.get(COMPONENT_MODIFIED));
-
-		cd.implementation = analyzer.getTypeRefFromFQN(impl == null ? name : impl);
+		
+		cd.implementation = analyzer.getTypeRefFromFQN(impl == null? name: impl);
+		
 
 		String provides = info.get(COMPONENT_PROVIDE);
 		if (info.get(COMPONENT_SERVICEFACTORY) != null) {
 			if (provides != null)
-				cd.scope = Boolean.valueOf(info.get(COMPONENT_SERVICEFACTORY)) ? ServiceScope.BUNDLE
-					: ServiceScope.SINGLETON;
+			    cd.servicefactory = Boolean.valueOf(info.get(COMPONENT_SERVICEFACTORY));
 			else
 				warning("The servicefactory:=true directive is set but no service is provided, ignoring it");
 		}
 
-		if (cd.scope == ServiceScope.BUNDLE && cd.immediate != null && cd.immediate) {
+		if (cd.servicefactory != null && cd.servicefactory  && cd.immediate != null && cd.immediate) {
 			// TODO can become error() if it is up to me
-			warning(
-				"For a Service Component, the immediate option and the servicefactory option are mutually exclusive for %s(%s)",
-				name, impl);
+			warning("For a Service Component, the immediate option and the servicefactory option are mutually exclusive for %(%s)",
+					name, impl);
 		}
-
-		// analyze the class for suitable methods.
-		final Map<String, MethodDef> lifecycleMethods = new HashMap<>();
-		final Map<String, MethodDef> bindmethods = new HashMap<>();
+		
+		//analyze the class for suitable methods.
+		final Map<String, MethodDef> lifecycleMethods = new HashMap<String, MethodDef>();
+		final Map<String, MethodDef> bindmethods = new HashMap<String, MethodDef>();
 		TypeRef typeRef = analyzer.getTypeRefFromFQN(impl);
 		Clazz clazz = analyzer.findClass(typeRef);
 		boolean privateAllowed = true;
-		boolean defaultAllowed = true;
-		String topPackage = typeRef.getPackageRef()
-			.getFQN();
+		boolean defaultAllowed = true; 
+		String topPackage = typeRef.getPackageRef().getFQN();
 		while (clazz != null) {
 			final boolean pa = privateAllowed;
 			final boolean da = defaultAllowed;
-			final Map<String, MethodDef> classLifecyclemethods = new HashMap<>();
-			final Map<String, MethodDef> classBindmethods = new HashMap<>();
-
+			final Map<String, MethodDef> classLifecyclemethods = new HashMap<String, MethodDef>();
+			final Map<String, MethodDef> classBindmethods = new HashMap<String, MethodDef>();
+			
 			clazz.parseClassFileWithCollector(new ClassDataCollector() {
-
+				
 				@Override
 				public void method(MethodDef md) {
 					Set<String> allowedParams = allowed;
 					String lifecycleName = null;
-
-					boolean isLifecycle = (cd.activate == null ? "activate" : cd.activate).equals(md.getName())
-						|| md.getName()
-							.equals(cd.modified);
-					if (!isLifecycle && (cd.deactivate == null ? "deactivate" : cd.deactivate).equals(md.getName())) {
+					
+					boolean isLifecycle = (cd.activate == null? "activate": cd.activate).equals(md.getName()) ||
+						md.getName().equals(cd.modified);	
+					if (!isLifecycle && (cd.deactivate == null? "deactivate": cd.deactivate).equals(md.getName())) {
 						isLifecycle = true;
 						allowedParams = allowedDeactivate;
 					}
-					if (isLifecycle && !lifecycleMethods.containsKey(md.getName())
-						&& (md.isPublic() || md.isProtected() || (md.isPrivate() && pa) || (!md.isPrivate()) && da)
-						&& isBetter(md, classLifecyclemethods.get(md.getName()), allowedParams)) {
+					if (isLifecycle && !lifecycleMethods.containsKey(md.getName()) &&
+							(md.isPublic() ||
+									md.isProtected() ||
+									(md.isPrivate() && pa) ||
+									(!md.isPrivate()) && da) &&
+							isBetter(md, classLifecyclemethods.get(md.getName()), allowedParams)) {
 						classLifecyclemethods.put(md.getName(), md);
 					}
-					if (!bindmethods.containsKey(md.getName())
-						&& (md.isPublic() || md.isProtected() || (md.isPrivate() && pa) || (!md.isPrivate()) && da)
-						&& isBetterBind(md, classBindmethods.get(md.getName()))) {
+					if (!bindmethods.containsKey(md.getName()) &&
+							(md.isPublic() ||
+									md.isProtected() ||
+									(md.isPrivate() && pa) ||
+									(!md.isPrivate()) && da) &&
+							isBetterBind(md, classBindmethods.get(md.getName()))) {
 						classBindmethods.put(md.getName(), md);
 					}
 				}
@@ -137,7 +126,7 @@ public class HeaderReader extends Processor {
 						return testRating < 6;// ignore invalid methods
 					if (testRating < rateLifecycle(existing, allowedParams))
 						return true;
-
+					
 					return false;
 				}
 
@@ -147,7 +136,7 @@ public class HeaderReader extends Processor {
 						return testRating < 6;// ignore invalid methods
 					if (testRating < rateBind(existing))
 						return true;
-
+					
 					return false;
 				}
 
@@ -159,30 +148,27 @@ public class HeaderReader extends Processor {
 				break;
 			clazz = analyzer.findClass(typeRef);
 			privateAllowed = false;
-			defaultAllowed = defaultAllowed && topPackage.equals(typeRef.getPackageRef()
-				.getFQN());
+			defaultAllowed = defaultAllowed && topPackage.equals(typeRef.getPackageRef().getFQN());
 		}
-
+		
+		
 		if (cd.activate != null && !lifecycleMethods.containsKey(cd.activate)) {
-			error("in component %s, activate method %s specified but not found", cd.implementation.getFQN(),
-				cd.activate);
+			error("in component %s, activate method %s specified but not found", cd.implementation.getFQN(), cd.activate);
 			cd.activate = null;
 		}
 		if (cd.deactivate != null && !lifecycleMethods.containsKey(cd.deactivate)) {
-			error("in component %s, deactivate method %s specified but not found", cd.implementation.getFQN(),
-				cd.deactivate);
+			error("in component %s, deactivate method %s specified but not found", cd.implementation.getFQN(), cd.deactivate);
 			cd.activate = null;
 		}
 		if (cd.modified != null && !lifecycleMethods.containsKey(cd.modified)) {
-			error("in component %s, modified method %s specified but not found", cd.implementation.getFQN(),
-				cd.modified);
+			error("in component %s, modified method %s specified but not found", cd.implementation.getFQN(), cd.modified);
 			cd.activate = null;
 		}
-
+		
 		provide(cd, provides, impl);
 		properties(cd, info, name);
 		reference(info, impl, cd, bindmethods);
-		// compute namespace after references, an updated method means ds 1.2.
+		//compute namespace after references, an updated method means ds 1.2.
 		getNamespace(info, cd, lifecycleMethods);
 		cd.prepare(analyzer);
 		return cd.getTag();
@@ -192,7 +178,8 @@ public class HeaderReader extends Processor {
 	private String checkIdentifier(String name, String value) {
 		if (value != null) {
 			if (!Verifier.isIdentifier(value)) {
-				error("Component attribute %s has value %s but is not a Java identifier", name, value);
+				error("Component attribute %s has value %s but is not a Java identifier",
+						name, value);
 				return null;
 			}
 		}
@@ -201,12 +188,13 @@ public class HeaderReader extends Processor {
 
 	/**
 	 * Check if we need to use the v1.1 namespace (or later).
-	 *
+	 * 
 	 * @param info
 	 * @param cd TODO
 	 * @param descriptors TODO
+	 * @return
 	 */
-	private void getNamespace(Map<String, String> info, ComponentDef cd, Map<String, MethodDef> descriptors) {
+	private void getNamespace(Map<String, String> info, ComponentDef cd, Map<String,MethodDef> descriptors) {
 		String namespace = info.get(COMPONENT_NAMESPACE);
 		if (namespace != null) {
 			cd.xmlns = namespace;
@@ -215,38 +203,38 @@ public class HeaderReader extends Processor {
 		if (version != null) {
 			try {
 				Version v = new Version(version);
-				cd.updateVersion(v, "set XML namespace version: " + version);
+				cd.updateVersion(v);
 			} catch (Exception e) {
-				error("version: specified on component header but not a valid version: %s", version);
+				error("version: specified on component header but not a valid version: "
+						+ version);
 				return;
 			}
 		}
 		for (String key : info.keySet()) {
 			if (SET_COMPONENT_DIRECTIVES_1_2.contains(key)) {
-				cd.updateVersion(V1_2, "uses 1.2 component directives like " + SET_COMPONENT_DIRECTIVES_1_2);
+				cd.updateVersion(AnnotationReader.V1_2);
 				return;
 			}
 		}
-		for (ReferenceDef rd : cd.references.values()) {
+		for (ReferenceDef rd: cd.references.values()) {
 			if (rd.updated != null) {
-				cd.updateVersion(V1_2, "updated");
+				cd.updateVersion(AnnotationReader.V1_2);
 				return;
 			}
 		}
-		// among other things this picks up any specified lifecycle methods
+		//among other things this picks up any specified lifecycle methods
 		for (String key : info.keySet()) {
 			if (SET_COMPONENT_DIRECTIVES_1_1.contains(key)) {
-				cd.updateVersion(V1_1, "1.1 component directives like " + SET_COMPONENT_DIRECTIVES_1_1);
+				cd.updateVersion(AnnotationReader.V1_1);
 				return;
 			}
 		}
-		for (String lifecycle : LIFECYCLE_METHODS) {
-			// lifecycle methods were not specified.... check for non 1.0
-			// signatures.
+		for (String lifecycle: LIFECYCLE_METHODS) {
+			//lifecycle methods were not specified.... check for non 1.0 signatures.
 			MethodDef test = descriptors.get(lifecycle);
-			if (descriptors.containsKey(lifecycle) && (!(test.isPublic() || test.isProtected())
-				|| rateLifecycle(test, "deactivate".equals(lifecycle) ? allowedDeactivate : allowed) > 1)) {
-				cd.updateVersion(V1_1, "base lifecyle methods " + LIFECYCLE_METHODS);
+			if (descriptors.containsKey(lifecycle) && (!(test.isPublic() || test.isProtected()) || 
+					rateLifecycle(test, "deactivate".equals(lifecycle)? allowedDeactivate: allowed) > 1)) {
+				cd.updateVersion(AnnotationReader.V1_1);
 				return;
 			}
 		}
@@ -254,7 +242,7 @@ public class HeaderReader extends Processor {
 
 	/**
 	 * Print the Service-Component properties element
-	 *
+	 * 
 	 * @param cd
 	 * @param info
 	 */
@@ -264,21 +252,15 @@ public class HeaderReader extends Processor {
 			Matcher m = PROPERTY_PATTERN.matcher(p);
 
 			if (m.matches()) {
-				String key = m.group(2);
-				String type = m.group(4);
-				if (type == null)
-					type = "String"; // default
-				String value = m.group(5);
+				String key = m.group(1).replaceAll("@", ":");
+				String value = m.group(4);
 				String parts[] = value.split("\\s*(\\||\\n)\\s*");
-				if (parts.length == 1 && value.endsWith("|")) {
-					String v = parts[0];
-					parts = new String[2];
-					parts[0] = v;
-					parts[1] = PropertyDef.MARKER;
+				for (String part: parts) {
+					cd.property.add(key, part);
 				}
-				cd.property.addProperty(key, type, parts);
 			} else
-				throw new IllegalArgumentException("Malformed property '" + p + "' on component: " + name);
+				throw new IllegalArgumentException("Malformed property '" + p
+						+ "' on component: " + name);
 		}
 	}
 
@@ -289,42 +271,39 @@ public class HeaderReader extends Processor {
 	void provide(ComponentDef cd, String provides, String impl) {
 		if (provides != null) {
 			StringTokenizer st = new StringTokenizer(provides, ",");
-			List<TypeRef> provide = new ArrayList<>();
+			List<TypeRef> provide = new ArrayList<TypeRef>();
 			while (st.hasMoreTokens()) {
 				String interfaceName = st.nextToken();
 				TypeRef ref = analyzer.getTypeRefFromFQN(interfaceName);
 				provide.add(ref);
-				analyzer.nonClassReferTo(ref);
+				analyzer.referTo(ref);
 
 				// TODO verifies the impl. class extends or implements the
 				// interface
 			}
-			cd.service = provide.toArray(new TypeRef[0]);
-		}
+			cd.service = provide.toArray(new TypeRef[provide.size()]);
+		} 
 	}
 
-	public final static Pattern REFERENCE = Pattern.compile("([^(]+)(\\(.+\\))?");
+	public final static Pattern	REFERENCE	= Pattern.compile("([^(]+)(\\(.+\\))?");
 
 	/**
-	 * rates the methods according to the scale in 112.5.8 (compendium 4.3, ds
-	 * 1.2), also returning "6" for invalid methods We don't look at return
-	 * values yet due to proposal to all them for setting service properties.
-	 *
+	 * rates the methods according to the scale in 112.5.8 (compendium 4.3, ds 1.2), also returning "6" for invalid methods
+	 * We don't look at return values yet due to proposal to all them for setting service properties.
 	 * @param test methodDef to examine for suitability as a DS lifecycle method
 	 * @param allowedParams TODO
 	 * @return rating; 6 if invalid, lower is better
 	 */
 	int rateLifecycle(MethodDef test, Set<String> allowedParams) {
-		TypeRef[] prototype = test.getDescriptor()
-			.getPrototype();
+		TypeRef[] prototype = test.getDescriptor().getPrototype();
 		if (prototype.length == 1 && ComponentContextTR.equals(prototype[0].getFQN()))
-			return 1;
+			    return 1;
 		if (prototype.length == 1 && BundleContextTR.equals(prototype[0].getFQN()))
 			return 2;
 		if (prototype.length == 1 && MapTR.equals(prototype[0].getFQN()))
 			return 3;
 		if (prototype.length > 1) {
-			for (TypeRef tr : prototype) {
+			for (TypeRef tr: prototype) {
 				if (!allowedParams.contains(tr.getFQN()))
 					return 6;
 			}
@@ -337,14 +316,12 @@ public class HeaderReader extends Processor {
 	}
 
 	/**
-	 * see 112.3.2. We can't distinguish the bind type, so we just accept
-	 * anything.
-	 *
+	 * see 112.3.2.  We can't distinguish the bind type, so we just accept anything.
 	 * @param test
+	 * @return
 	 */
 	int rateBind(MethodDef test) {
-		TypeRef[] prototype = test.getDescriptor()
-			.getPrototype();
+		TypeRef[] prototype = test.getDescriptor().getPrototype();
 		if (prototype.length == 1 && ServiceReferenceTR.equals(prototype[0].getFQN()))
 			return 1;
 		if (prototype.length == 1)
@@ -354,12 +331,19 @@ public class HeaderReader extends Processor {
 		return 6;
 	}
 
-	void reference(Map<String, String> info, String impl, ComponentDef cd, Map<String, MethodDef> descriptors)
-		throws Exception {
-		Collection<String> dynamic = new ArrayList<>(split(info.get(COMPONENT_DYNAMIC)));
-		Collection<String> optional = new ArrayList<>(split(info.get(COMPONENT_OPTIONAL)));
-		Collection<String> multiple = new ArrayList<>(split(info.get(COMPONENT_MULTIPLE)));
-		Collection<String> greedy = new ArrayList<>(split(info.get(COMPONENT_GREEDY)));
+	/**
+	 * @param info
+	 * @param impl TODO
+	 * @param descriptors TODO
+	 * @param pw
+	 * @throws Exception 
+	 */
+	void reference(Map<String, String> info, String impl, ComponentDef cd, Map<String,MethodDef> descriptors) throws Exception {
+		Collection<String> dynamic = new ArrayList<String>(split(info.get(COMPONENT_DYNAMIC)));
+		Collection<String> optional = new ArrayList<String>(split(info.get(COMPONENT_OPTIONAL)));
+		Collection<String> multiple = new ArrayList<String>(split(info.get(COMPONENT_MULTIPLE)));
+		Collection<String> greedy = new ArrayList<String>(split(info.get(COMPONENT_GREEDY)));
+
 
 		for (Map.Entry<String, String> entry : info.entrySet()) {
 
@@ -367,7 +351,8 @@ public class HeaderReader extends Processor {
 			String referenceName = entry.getKey();
 			if (referenceName.endsWith(":")) {
 				if (!SET_COMPONENT_DIRECTIVES.contains(referenceName))
-					error("Unrecognized directive in " + Constants.SERVICE_COMPONENT + " header: %s", referenceName);
+					error("Unrecognized directive in Service-Component header: "
+							+ referenceName);
 				continue;
 			}
 
@@ -390,9 +375,9 @@ public class HeaderReader extends Processor {
 				} else {
 					bind = calculateBind(referenceName);
 				}
-				bind = parts[1].length() == 0 ? calculateBind(referenceName) : parts[1];
+				bind = parts[1].length() == 0? calculateBind(referenceName): parts[1];
 				if (parts.length > 2 && parts[2].length() > 0) {
-					unbind = parts[2];
+					unbind = parts[2] ;
 					unbindCalculated = false;
 				} else {
 					if (bind.startsWith("add"))
@@ -407,13 +392,14 @@ public class HeaderReader extends Processor {
 			} else if (Character.isLowerCase(referenceName.charAt(0))) {
 				bind = calculateBind(referenceName);
 				unbind = "un" + bind;
-				updated = "updated" + Character.toUpperCase(referenceName.charAt(0)) + referenceName.substring(1);
+				updated = "updated" + Character.toUpperCase(referenceName.charAt(0))
+				+ referenceName.substring(1);
 			}
 
 			String interfaceName = entry.getValue();
 			if (interfaceName == null || interfaceName.length() == 0) {
-				error("Invalid Interface Name for references in Service Component: %s=%s", referenceName,
-					interfaceName);
+				error("Invalid Interface Name for references in Service Component: "
+						+ referenceName + "=" + interfaceName);
 				continue;
 			}
 
@@ -425,8 +411,7 @@ public class HeaderReader extends Processor {
 					if (bindCalculated)
 						bind = null;
 					else
-						error("In component %s, the bind method %s for %s not defined", cd.effectiveName(), bind,
-							referenceName);
+						error("In component %s, the bind method %s for %s not defined", cd.name, bind, referenceName);
 
 				// Check if the unbind method exists
 				if (!descriptors.containsKey(unbind)) {
@@ -434,16 +419,14 @@ public class HeaderReader extends Processor {
 						// remove it
 						unbind = null;
 					else
-						error("In component %s, the unbind method %s for %s not defined", cd.effectiveName(), unbind,
-							referenceName);
+						error("In component %s, the unbind method %s for %s not defined", cd.name, unbind, referenceName);
 				}
 				if (!descriptors.containsKey(updated)) {
 					if (updatedCalculated)
-						// remove it
+						//remove it
 						updated = null;
-					else
-						error("In component %s, the updated method %s for %s is not defined", cd.effectiveName(),
-							updated, referenceName);
+					else 
+						error("In component %s, the updated method %s for %s is not defined", cd.name, updated, referenceName);
 				}
 			}
 			// Check the cardinality by looking at the last
@@ -468,8 +451,8 @@ public class HeaderReader extends Processor {
 				target = m.group(2);
 			}
 			TypeRef ref = analyzer.getTypeRefFromFQN(interfaceName);
-			analyzer.nonClassReferTo(ref);
-			ReferenceDef rd = new ReferenceDef(null);
+			analyzer.referTo(ref);
+			ReferenceDef rd = new ReferenceDef();
 			rd.name = referenceName;
 			rd.service = interfaceName;
 
@@ -499,12 +482,9 @@ public class HeaderReader extends Processor {
 			if (dynamic.contains(referenceName)) {
 				rd.policy = ReferencePolicy.DYNAMIC;
 				if (rd.unbind == null)
-					error("In component %s, reference %s is dynamic but has no unbind method.", cd.effectiveName(),
-						rd.name)
-							.details(new DeclarativeServicesAnnotationError(cd.implementation.getFQN(), null, null,
-								ErrorType.DYNAMIC_REFERENCE_WITHOUT_UNBIND));
+					error("In component %s, reference %s is dynamic but has no unbind method.", cd.name, rd.name);
 			}
-
+			
 			if (greedy.contains(referenceName)) {
 				rd.policyOption = ReferencePolicyOption.GREEDY;
 			}
@@ -517,7 +497,8 @@ public class HeaderReader extends Processor {
 	}
 
 	private String calculateBind(String referenceName) {
-		return "set" + Character.toUpperCase(referenceName.charAt(0)) + referenceName.substring(1);
+		return "set" + Character.toUpperCase(referenceName.charAt(0))
+		+ referenceName.substring(1);
 	}
 
 }
